"""Wall objects."""
from circadapt.components import Component


def rebuild_patch2022(model):
    """Rebuilts patches of the model, e.g. for when n_patch changed."""
    model['Patch2022'].build()


class Wall2022(Component):
    """Wall2022 object."""

    parameters = [
        'AmDead',
        'nPatch',
        ]
    parameter_on_set = {
        # default patch2022, overwrite when other patches are used
        'nPatch': rebuild_patch2022
        }

    signals = [
        'A',
        'Am',
        'Am0',
        'Cm',
        'DADT',
        'p_trans',
        'T',
        'Vm',
        'VWall',
        'Z',
        ]


class PreAfterloadExperiment(Component):
    """
    PreAfterloadExperiment object.

    Parameters
    ==========
    AmDead: float
        Non-conctractile wall area.
    AmRefAfterload: float
        Wall area at :math:`T_{m} = Tafterload`.
    nPatch: int
        Number of patches. On change, the last patch is splitted or last
        patches are removed.
    nIter: int
        Number of iterations to calculate mechanical stability.
    AmRefAfterload: float
        Maximum wall tension. When tension is reached, wall area shortens.

    Signals
    =======
    Signals are arrays. Each point in the array represents a point in time with
    step-size controlled by the solver.

    Am: array
        Wall area
    Am0: array
        Linearized stress-free wall area
    Cm: array
        Wall curvature
    DADT: array
        Stress-tension derivative
    T: array
        Tension
    p_trans: array
        Transmural pressure
    """
    parameters = [
        'AmDead',
        'AmRefAfterload',
        'nPatch',
        'nIter',
        'Tafterload',
        ]
    parameter_on_set = {
        'nPatch': rebuild_patch2022,
        }
    signals = [
        'A',
        'Am',
        'Am0',
        'Cm',
        'DADT',
        'p_trans',
        'T',
        'Vm',
        'VWall',
        'Z',
        ]
