def strip_base(base):
    if base is None:
        base = ''
    elif len(base)>0 and base[-1]!='.':
        base = base + '.'
    return base


def build_heart(self, base=None, options=None):
    _options = {'build': 'Walmsley2015',
                'patch_type': None,
                'PuArt': 'PuArt',
                'SyVen': 'SyVen',
                'PuVen': 'PuVen',
                'SyArt': 'SyArt'}
    if options is not None:
        _options.update(options)
    options = _options

    # defaults
    if options['patch_type'] is None and options['build'] == 'Walmsley2015':
        options['patch_type'] = 'Patch'
    elif options['patch_type'] in ['Patch']:
        pass
    else:
        raise ValueError(' patch_type ' + options['patch_type'] + ' not known.')

    base = strip_base(base)

    # Create components
    self.add_component('Bag', 'Peri', base[:-1])
    self.add_component('TriSeg', 'TriSeg', base+'Peri')
    self.add_component(options['patch_type'], 'pLv1', base+'Peri.TriSeg.wLv')
    self.add_component(options['patch_type'], 'pSv1', base+'Peri.TriSeg.wSv')
    self.add_component(options['patch_type'], 'pRv1', base+'Peri.TriSeg.wRv')

    self.add_component('Chamber', 'La', base+'Peri')
    self.add_component(options['patch_type'], 'pLa1', base+'Peri.La.W')
    self.add_component('Chamber', 'Ra', base+'Peri')
    self.add_component(options['patch_type'], 'pRa1', base+'Peri.Ra.W')

    self.add_component('Valve', 'SyVenRa', base+'Peri')
    self.add_component('Valve', 'RaRv', base+'Peri')
    self.add_component('Valve', 'RvPuArt', base+'Peri')
    self.add_component('Valve', 'PuVenLa', base+'Peri')
    self.add_component('Valve', 'LaLv', base+'Peri')
    self.add_component('Valve', 'LvSyArt', base+'Peri')

    # Link components
    self.set_component('Peri.SyVenRa.Prox', options['SyVen'])
    self.set_component('Peri.SyVenRa.Dist', 'Peri.Ra')
    self.set_component('Peri.RaRv.Prox', 'Peri.Ra')
    self.set_component('Peri.RaRv.Dist', 'Peri.TriSeg.cRv')
    self.set_component('Peri.RvPuArt.Prox', 'Peri.TriSeg.cRv')
    self.set_component('Peri.RvPuArt.Dist', options['PuArt'])

    self.set_component('Peri.PuVenLa.Prox', options['PuVen'])
    self.set_component('Peri.PuVenLa.Dist', 'Peri.La')
    self.set_component('Peri.LaLv.Prox', 'Peri.La')
    self.set_component('Peri.LaLv.Dist', 'Peri.TriSeg.cLv')
    self.set_component('Peri.LvSyArt.Prox', 'Peri.TriSeg.cLv')
    self.set_component('Peri.LvSyArt.Dist', options['SyArt'])

    # Parameterize components
    self.set('Model.'+base+'Peri.TriSeg.VS', 44e-6)
    self.set('Model.'+base+'Peri.TriSeg.YS', 34.6e-3)

    self.set('Model.'+base+'Peri.TriSeg.cLv.V',  100e-6)
    self.set('Model.'+base+'Peri.TriSeg.cRv.V',  100e-6)
    self.set('Model.'+base+'Peri.La.V',  50e-6)
    self.set('Model.'+base+'Peri.Ra.V',  50e-6)

    set_parameters_walmsley2015_heart(self, base)

def set_parameters_walmsley2015_heart(self, base):
    # all wall parameters
    locs = ['Peri.TriSeg.wLv.pLv1',
            'Peri.TriSeg.wSv.pSv1',
            'Peri.TriSeg.wRv.pRv1',
            'Peri.La.W.pLa1',
            'Peri.Ra.W.pRa1',
            ]
    for loc in locs:
        self.set('Model.'+base+loc+'.LenSeriesElement', 0.04)
        self.set('Model.'+base+loc+'.LsRef', 2.0)
        self.set('Model.'+base+loc+'.Ls0Pas', 1.8)
        self.set('Model.'+base+loc+'.dLsPas', 0.6)
        self.set('Model.'+base+loc+'.k1', 10)
        self.set('Model.'+base+loc+'.dT', 0)
        self.set('Model.'+base+loc+'.CRest', 0)
        self.set('Model.'+base+loc+'.Lsi0Act', 1.51)
        self.set('Model.'+base+loc+'.LDAD', 1.057)
        self.set('Model.'+base+loc+'.ADO', 0.65)
        self.set('Model.'+base+loc+'.LDCI', 9.1204)

    # wall specific
    self.set('Model.'+base+'Peri.TriSeg.wLv.pLv1.AmRef', 0.009805314057621018)
    self.set('Model.'+base+'Peri.TriSeg.wLv.pLv1.VWall', 9.60080108859341e-05)
    self.set('Model.'+base+'Peri.TriSeg.wLv.pLv1.vMax', 7.0)
    self.set('Model.'+base+'Peri.TriSeg.wLv.pLv1.SfPas', 22311)
    self.set('Model.'+base+'Peri.TriSeg.wLv.pLv1.TR', 0.25)
    self.set('Model.'+base+'Peri.TriSeg.wLv.pLv1.TD', 0.25)
    self.set('Model.'+base+'Peri.TriSeg.wLv.pLv1.SfAct', 120000)

    self.set('Model.'+base+'Peri.TriSeg.wSv.pSv1.AmRef', 0.004884229725474014)
    self.set('Model.'+base+'Peri.TriSeg.wSv.pSv1.VWall', 3.23553932981624e-05)
    self.set('Model.'+base+'Peri.TriSeg.wSv.pSv1.vMax', 7.0)
    self.set('Model.'+base+'Peri.TriSeg.wSv.pSv1.SfPas', 22086)
    self.set('Model.'+base+'Peri.TriSeg.wSv.pSv1.TR', 0.25)
    self.set('Model.'+base+'Peri.TriSeg.wSv.pSv1.TD', 0.25)
    self.set('Model.'+base+'Peri.TriSeg.wSv.pSv1.SfAct', 120000)

    self.set('Model.'+base+'Peri.TriSeg.wRv.pRv1.AmRef', 0.012970820071535782)
    self.set('Model.'+base+'Peri.TriSeg.wRv.pRv1.VWall', 6.29708653593736e-05)
    self.set('Model.'+base+'Peri.TriSeg.wRv.pRv1.vMax', 7.0)
    self.set('Model.'+base+'Peri.TriSeg.wRv.pRv1.SfPas', 22763)
    self.set('Model.'+base+'Peri.TriSeg.wRv.pRv1.TR', 0.25)
    self.set('Model.'+base+'Peri.TriSeg.wRv.pRv1.TD', 0.25)
    self.set('Model.'+base+'Peri.TriSeg.wRv.pRv1.SfAct', 120000)

    self.set('Model.'+base+'Peri.La.W.pLa1.AmRef', 0.006921646911242356)
    self.set('Model.'+base+'Peri.La.W.pLa1.VWall', 1.5582632867621123e-05)
    self.set('Model.'+base+'Peri.La.W.pLa1.vMax', 14.0)
    self.set('Model.'+base+'Peri.La.W.pLa1.SfPas', 51015)
    self.set('Model.'+base+'Peri.La.W.pLa1.TR', 0.4)
    self.set('Model.'+base+'Peri.La.W.pLa1.TD', 0.4)
    self.set('Model.'+base+'Peri.La.W.pLa1.SfAct', 84000)

    self.set('Model.'+base+'Peri.Ra.W.pRa1.AmRef', 0.0059862195992063825)
    self.set('Model.'+base+'Peri.Ra.W.pRa1.VWall', 6.389451836068464e-05)
    self.set('Model.'+base+'Peri.Ra.W.pRa1.vMax', 14.0)
    self.set('Model.'+base+'Peri.Ra.W.pRa1.SfPas', 52473)
    self.set('Model.'+base+'Peri.Ra.W.pRa1.TR', 0.4)
    self.set('Model.'+base+'Peri.Ra.W.pRa1.TD', 0.4)
    self.set('Model.'+base+'Peri.Ra.W.pRa1.SfAct', 84000)

def build_timings(self, base=None, options=None):
    _options = {'build': 'Walmsley2015',
                }
    if options is not None:
        _options.update(options)
    options = _options

    base = strip_base(base)

    if options['build'] == 'Walmsley2015':
        self.add_component('Timings', 'Timings', base[:-1]+'')

        self.set_component('Timings.wLa', 'Peri.La.W')
        self.set_component('Timings.wRa', 'Peri.Ra.W')
        self.set_component('Timings.wLv', 'Peri.TriSeg.wLv')
        self.set_component('Timings.wSv', 'Peri.TriSeg.wSv')
        self.set_component('Timings.wRv', 'Peri.TriSeg.wRv')

def build_pfc(self, base=None, options=None):
    _options = {'build': 'Walmsley2015',
                }
    if options is not None:
        _options.update(options)
    options = _options

    base = strip_base(base)
    if options['build'] == 'Walmsley2015':
        self.add_component('PressureFlowControl', 'PFC', base[:-1]+'')

        self.set_component('PFC.cavPressure', 'SyArt')
        self.set_component('PFC.valveReturn', 'Peri.SyVenRa')
        self.set_component('PFC.ArtVenResistance', 'CiSy')
        self.set_component('PFC.valveFlowVec', 'Peri.SyVenRa')
        self.set_component('PFC.valveFlowVec', 'Peri.RaRv')
        self.set_component('PFC.valveFlowVec', 'Peri.RvPuArt')
        self.set_component('PFC.valveFlowVec', 'Peri.PuVenLa')
        self.set_component('PFC.valveFlowVec', 'Peri.LaLv')
        self.set_component('PFC.valveFlowVec', 'Peri.LvSyArt')
